package dev.fyloz.backup.logic

import dev.fyloz.backup.data.FileProvider
import dev.fyloz.backup.dtos.CreateBackupResponse
import dev.fyloz.backup.dtos.UploadFileResponse
import dev.fyloz.backup.entities.Backup
import dev.fyloz.backup.entities.BackupFile
import dev.fyloz.backup.entities.Machine
import dev.fyloz.backup.exceptions.NotFoundException
import dev.fyloz.backup.exceptions.ValidationException
import dev.fyloz.backup.modules.backup.AddBackupFileRequestBody
import dev.fyloz.backup.repositories.BackupRepository
import dev.fyloz.backup.repositories.MachineRepository
import dev.fyloz.backup.utils.HashUtils
import io.ktor.http.content.*
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject
import org.litote.kmongo.id.ObjectIdGenerator.generateNewId
import org.litote.kmongo.id.StringId
import java.time.LocalDateTime

interface BackupLogic {
    fun create(machineId: String): CreateBackupResponse
    fun addFile(id: String, murmur3Hash: String, body: AddBackupFileRequestBody): UploadFileResponse
}

class DefaultBackupLogic : BackupLogic, KoinComponent {
    private val repository by inject<BackupRepository>()
    private val machineRepository by inject<MachineRepository>()
    private val fileProvider by inject<FileProvider>()

    override fun create(machineId: String): CreateBackupResponse {
        val backup = Backup(generateNewId())

        val machine = machineRepository.findById(machineId)
        val updatedMachine = machine?.copy(backups = machine.backups + backup) ?: Machine(
            StringId(machineId),
            setOf(backup),
            LocalDateTime.now()
        )

        repository.save(backup)
        machineRepository.save(updatedMachine)

        return CreateBackupResponse(backup.id.toString(), machineId)
    }

    override fun addFile(id: String, murmur3Hash: String, body: AddBackupFileRequestBody): UploadFileResponse {
        val backup = repository.findById(id) ?: throw NotFoundException("Could not find a backup with the id '$id'")

        saveFile(body.file, murmur3Hash)

        val file = BackupFile(generateNewId(), body.originalName.value, body.path.value, murmur3Hash)

        val updatedBackup = backup.copy(files = backup.files + file)
        repository.save(updatedBackup)

        return UploadFileResponse(file)
    }

    private fun saveFile(file: PartData.FileItem, murmur3Hash: String) {
        val data = file.streamProvider().readAllBytes()
        if (!HashUtils.murmur3Matches(data, murmur3Hash)) {
            // File is corrupted, throw
            throw ValidationException("Corrupted file, hash did not match")
        }

        fileProvider.save(data, "/home/william/Dev/Projects/backups/server/data/test.dat")
    }
}