package dev.fyloz.backup.modules.backup

import dev.fyloz.backup.Constants
import dev.fyloz.backup.logic.BackupLogic
import dev.fyloz.backup.modules.backup.requests.CreateBackupRequest
import dev.fyloz.backup.utils.validateAndGetHeader
import io.ktor.http.content.*
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import org.koin.ktor.ext.inject

fun Route.backupModule() {
    val logic by inject<BackupLogic>()

    route("/backup") {
        post("create") {
            val request = call.receive<CreateBackupRequest>()
            val response = logic.create(request.machineId)

            call.respond(response)

            // Start a new backup
            // Returns a backup id and a public key to sign the data

            // https://stackoverflow.com/questions/40243857/how-to-encrypt-large-file-with-rsa
        }

        post("finish/:id") {
            // Finishes the backup with the given id
            // Finished backups cannot be edited
        }

        put("/{${Constants.RequestParameters.ID}}") {
            val id = call.parameters[Constants.RequestParameters.ID]!!
            val murmurHash3 = validateAndGetHeader(Constants.RequestHeaders.CONTENT_MURMUR3)

            val multipartData = call.receiveMultipart().readAllParts()
            val body = AddBackupFileRequestBody(multipartData.associateBy { it.name!! })

            withContext(Dispatchers.IO) {
                val backup = logic.addFile(id, murmurHash3, body)
                call.respond(backup)
            }

            // Add a file to the backup with the given id
            // The client has to send the file along with a checksums
            // The file has to be encrypted with the public key created previously

            // For incremental backups, the server will create a diff with the previous version
        }

        delete("cancel/:id") {
            // Cancels the backup with the given id
            // Removes all the files and revokes the encryption key
        }

        delete("delete/:id") {
            // Deletes the backup with the given id
            // An incremental backup can only be deleted if it is the last one, as deleting previous ones will break the newest backups
            // The last normal backup cannot be deleted as the incremental backups are be based on it
        }
    }
}