package dev.fyloz.backup.repositories

import dev.fyloz.backup.data.MongoDatabase
import dev.fyloz.backup.entities.Backup
import org.bson.types.ObjectId
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject
import org.litote.kmongo.*
import org.litote.kmongo.id.toId
import kotlin.random.Random

interface BackupRepository {
    /** Returns all stored backups. */
    fun findAll(): Collection<Backup>

    /** Searches for a backup with the given [id] and returns it, if found. */
    fun findById(id: String): Backup?

    /** Saves the given [backup]. */
    fun save(backup: Backup)

    /** Deletes the backup with the given [id]. */
    fun deleteById(id: String)
}

class MongoBackupRepository : BackupRepository, KoinComponent {
    private val database by inject<MongoDatabase>()
    private val backups = database.backupCollection

    override fun findAll() = backups.find().toList()
    override fun findById(id: String): Backup? {
        val bsonId = ObjectId(id).toId<Backup>()
        return backups.findOneById(bsonId)
    }

    override fun save(backup: Backup) {
        backups.save(backup)
    }

    override fun deleteById(id: String) {
        backups.deleteOneById(id)
    }
}

class MemoryBackupRepository : BackupRepository {
    private val backups = mutableMapOf<String, Backup>()
    private val idCharPool = createIdCharPool()

    override fun findAll() = backups.values
    override fun findById(id: String) = backups[id]

    // Creates a BSON id look-a-like
    fun getNewId(): String {
        var id = generateId()

        while (id in backups) {
            id = generateId()
        }

        return generateId()
    }

    private fun generateId() = (1..ID_STRING_LENGTH)
        .map { Random.nextInt(idCharPool.size) }
        .map { idCharPool[it] }
        .joinToString("")

    override fun save(backup: Backup) {
        backups[backup.id.toString()] = backup
    }

    override fun deleteById(id: String) {
        backups.remove(id)
    }

    companion object {
        private const val ID_STRING_LENGTH = 16

        fun createIdCharPool() = (0x41..0x5a)
            .plus(0x61..0x7a)
            .map { it.toChar() }
    }
}